"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = require("path");
var errors_1 = require("./errors");
var merge_schemas_1 = require("./merge-schemas");
var validate_documents_1 = require("./loaders/documents/validate-documents");
function getPluginByName(name, pluginLoader) {
    return __awaiter(this, void 0, void 0, function () {
        var possibleNames, possibleModules, _i, possibleModules_1, moduleName, possibleNamesMsg;
        return __generator(this, function (_a) {
            possibleNames = [
                "graphql-codegen-" + name,
                "graphql-codegen-" + name + "-template",
                "codegen-" + name,
                "codegen-" + name + "-template",
                name
            ];
            possibleModules = possibleNames.concat(path_1.resolve(process.cwd(), name));
            for (_i = 0, possibleModules_1 = possibleModules; _i < possibleModules_1.length; _i++) {
                moduleName = possibleModules_1[_i];
                try {
                    return [2 /*return*/, pluginLoader(moduleName)];
                }
                catch (err) {
                    if (err.message.indexOf("Cannot find module '" + moduleName + "'") === -1) {
                        throw new errors_1.DetailedError("Unable to load template plugin matching " + name, "\n              Unable to load template plugin matching '" + name + "'.\n              Reason: \n                " + err.message + "\n            ");
                    }
                }
            }
            possibleNamesMsg = possibleNames
                .map(function (name) {
                return ("\n        - " + name + "\n    ").trimRight();
            })
                .join('');
            throw new errors_1.DetailedError("Unable to find template plugin matching " + name, "\n        Unable to find template plugin matching '" + name + "'\n        Install one of the following packages:\n        \n        " + possibleNamesMsg + "\n      ");
        });
    });
}
exports.getPluginByName = getPluginByName;
function executePlugin(options, pluginPackage) {
    return __awaiter(this, void 0, void 0, function () {
        var outputSchema, errors, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!pluginPackage || !pluginPackage.plugin || typeof pluginPackage.plugin !== 'function') {
                        throw new errors_1.DetailedError("Invalid Custom Plugin \"" + options.name + "\"", "\n        Plugin " + options.name + " does not export a valid JS object with \"plugin\" function.\n  \n        Make sure your custom plugin is written in the following form:\n  \n        module.exports = {\n          plugin: (schema, documents, config) => {\n            return 'my-custom-plugin-content';\n          },\n        };\n        ");
                    }
                    outputSchema = merge_schemas_1.buildSchema(options.schema);
                    if (outputSchema && options.documents.length > 0) {
                        errors = validate_documents_1.validateGraphQlDocuments(outputSchema, options.documents);
                        validate_documents_1.checkValidationErrors(errors);
                    }
                    if (!(pluginPackage.validate && typeof pluginPackage.validate === 'function')) return [3 /*break*/, 4];
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, pluginPackage.validate(outputSchema, options.documents, options.config, options.outputFilename, options.allPlugins)];
                case 2:
                    _a.sent();
                    return [3 /*break*/, 4];
                case 3:
                    e_1 = _a.sent();
                    throw new errors_1.DetailedError("Plugin \"" + options.name + "\" validation failed:", "\n            " + e_1.message + "\n          ");
                case 4: return [2 /*return*/, pluginPackage.plugin(outputSchema, options.documents, options.config)];
            }
        });
    });
}
exports.executePlugin = executePlugin;
//# sourceMappingURL=execute-plugin.js.map