"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var isGlob = require("is-glob");
var isValidPath = require("is-valid-path");
var graphql_1 = require("graphql");
var glob = require("glob");
var fs_1 = require("fs");
var fs = require("fs");
var errors_1 = require("../../errors");
var documents_from_glob_1 = require("../documents/documents-from-glob");
var epoxy_1 = require("@graphql-modules/epoxy");
function isGraphQLFile(globPath) {
    return documents_from_glob_1.graphQLExtensions.some(function (ext) { return globPath.endsWith(ext); });
}
function loadSchemaFile(filepath) {
    var content = fs.readFileSync(filepath, {
        encoding: 'utf-8'
    });
    if (/^\# import /i.test(content.trimLeft())) {
        var importSchema = require('graphql-import').importSchema;
        return importSchema(filepath);
    }
    return content;
}
var SchemaFromTypedefs = /** @class */ (function () {
    function SchemaFromTypedefs() {
    }
    SchemaFromTypedefs.prototype.canHandle = function (globPath) {
        return isGlob(globPath) || (isValidPath(globPath) && isGraphQLFile(globPath));
    };
    SchemaFromTypedefs.prototype.handle = function (globPath, config, schemaOptions) {
        var globFiles = glob.sync(globPath, { cwd: process.cwd() });
        if (!globFiles || globFiles.length === 0) {
            throw new errors_1.DetailedError('Unable to find matching files', "\n      \n        Unable to find matching files for glob: " + globPath + " in directory: " + process.cwd() + "\n      ");
        }
        if (globFiles.length > 1) {
            return epoxy_1.mergeGraphQLSchemas(globFiles.map(function (filePath) { return fs_1.readFileSync(filePath, 'utf-8'); }));
        }
        else {
            return graphql_1.parse(loadSchemaFile(globFiles[0]));
        }
    };
    return SchemaFromTypedefs;
}());
exports.SchemaFromTypedefs = SchemaFromTypedefs;
//# sourceMappingURL=schema-from-typedefs.js.map